@if (@CodeSection == @Batch) @then

:: The first line above is...
:: - in Batch: a valid IF command that does nothing.
:: - in JScript: a conditional compilation IF statement that is false,
::               so this Batch section is omitted until next "at-sign end".


@echo off

rem FindRepl.bat: Utility to search and search/replace strings in a file
rem http://www.dostips.com/forum/viewtopic.php?f=3&t=4697
rem Antonio Perez Ayala

rem   - Jun/26/2013: first version.
rem   - Jul/02/2013: use /Q in submatched substrings, use /VR for /V in Replace, eliminate "\r\n" in blocks.
rem   - Jul/07/2013: change /VR by /R, search for "\r\n" with /$ and no /V, /N nor blocks.

rem   - Nov/20/2014: Version 2 - New switches: /J, /L, /G, /ARG#, and || separate regexp's in /Alternations.

if "%~1" equ "/?" goto showUsage

Rem Modified from FindRepl.bat to intercept HELP when no arguments
if "%~1" equ "" goto GetSet

CScript //nologo //E:JScript "%~F0" %*
exit /B %errorlevel%

<usage>
Searches for strings in Stdin file, and prints or replaces them.

FINDREPL [/I] [/V] [/N] rSearch [/E:rEndBlk] [/O:s:e] [/B:rBlock] [/$:1:2...]
         [[/R] [/A] sReplace] [/Q:c] [/S:sSource]
         [/J[:n] [/L:jEnd]] [/G:file] [/ARG1:text1 [/ARG2:text2] ...]

  /I         Specifies that the search is not to be case-sensitive.
  /V         Prints only lines that do not contain a match.
  /N         Prints the line number before each line that matches.
  rSearch    Text to be searched for, or Start text for a block of lines.
  /E:rEndBlk Text to be searched for the End of a block of lines.
  /O:s:e     Specifies offsets for Starting and Ending lines of blocks.
  /B:rBlock  Text to be searched again in the blocks of lines.
  /$:1:2...  Specifies to print saved submatched substrings instead of lines.
  /R         Prints only replaced lines, instead of all file lines.
  /A         Specifies that sReplace has alternative values matching rSearch.
  sReplace   Text that will replace the matched text.
  /Q:c       Specifies a character that is used in place of quotation marks.
  /S:sSource Text to be processed instead of Stdin file.
  /J[:n]     Specify that rSearch/sReplace contain expressions (regexp/JScript)
  /L:jEnd    Execute jEnd as a JScript expression after the end of file.
  /G:file    Specifies the file to get rSearch and sReplace texts from.
  /ARG1:...  Specifies texts to initialize JScript variables with same names.

All search texts must be given in VBScript regular expression format.
The replacement string may use $ to retrieve saved submatched substrings. If /J
switch is given, the JScript expression in sReplace text use submatched $0-$n.
Use /A switch to insert alternatives separated by pipe in rSearch/sReplace;
if /J switch is given, the alternatives are regexp's separated by double-pipe.
Use /Q:c switch to insert a quote in the search/replacement texts.
If first character of any text is an equal-sign, specify a Batch variable.

There are three ways to define Blocks of lines using rSearch text as base:

/O:s:e                           /E:rEndBlk              /E:rEndBlk /O:s:e
-------------------------------------------------------------------------------
Add S and E (with optional       From rSearch line       Add S to rSearch line
signs) to matching lines.        to rEndBlk line.        and E to rEndBlk line.

... and one way more if rSearch is not given:   /O:s:e   From line S to line E.
If S or E is negative, specify a backwards line from the end of file.
If E is not given, then it defaults to the last line of the file (same as -1).

The output vary depending on the given parameters and switches this way:

rSearch        /V                 Block            /B:rBlock        /$:1:2...
-------------------------------------------------------------------------------
Matched        Non-matched        Blocks of        Search /B:       Saved
lines.         lines.             lines.           in blocks        submatches.

sReplace       /R                 Block            /B:rBlock
-------------------------------------------------------------------------------
All file       Only replaced      Search /B:rBlock in blocks
lines.         file lines.        and replaces matched lines.

The total number of matchings/replacements is returned in ERRORLEVEL.

   1- FindRepl.bat documentation:
         Detailed description of FindRepl features with multiple examples.
   2- FindRepl.bat version 2 documentation:
         Additional descriptions on /J, /L, /G switches and || alternatives.
   3- Regular Expressions documentation:
         Describe the features that may be used in rSearch, rEndBlk and rBlock.
   4- Alternation and Subexpressions documentation:
         Describe how use | to separate values in rSearch with /A switch
         and features of (subexpressions) for /$ switch and $0..$n in sReplace.
   5- JScript expressions documentation (/J switch):
         Describe the operators that may be used in JScript expressions.
   6- Data types and functions for JScript expressions
         Describe additional data types and functions in JScript:
         - String Object: functions to manipulate strings.
         - Math Object: arithmetic functions.
         - Date Object: functions for date calculations.
         See also topic 2- section 3- FindRepl.bat predefined functions.
   0- End this help screen.
</usage>

:GetSet
@echo off & setlocal enableextensions disabledelayedexpansion & Rem First LOCAL scope
Rem Start http://www.dostips.com/forum/viewtopic.php?p=54670#p54670
(set lf=^
%= BLANK LINE REQUIRED =%
)
set "cr=" & if not defined cr for /f "skip=1" %%C in (
'echo(^|replace ? . /w /u'
) do set "cr=%%C"

set ^"orig=  !random!  !  ^^!  ^^^^!  ^"^^  ^&^"^&  ^^^" %%os%%  ^"

call :strim res1 orig

@Echo Off&SetLOCAL EnableDelayedExpansion &Rem.Second LOCAL scope and keep environment uncluttered.

Rem Define fully qualified output batch script
Set "FireFile=%~dp0MMDls%~x0"
Rem Define short output batch script name for messages.
Set "LoadFile=MMDls%~x0"

Rem copy to user script from beginning up to end of help tag
< "%~f0" (
    CScript //nologo //E:JScript "%~F0" "CodeSection" /o:0:+1 /E:"^</usage>"
) > "%FireFile%"
    Echo=
    Echo=%~nx0 copied %errorlevel% records from itself to "%LoadFile%"
    >> "%FireFile%" Echo=%~nx0 copied %errorlevel% records from itself to "%LoadFile%"
    >> "%FireFile%" Echo=
    >> "%FireFile%" Echo=%~nx0 generated the following table records:
    >> "%FireFile%" Echo=

Rem Convert PDF file to a usable format temporary file
    Set "YY=18"
    Set "YYYY=20%YY%"
    Set "TeamFile=MMDLteams.txt"
    Echo.
    Echo.Converting input file "MMDL_S%YY%_SS_Sched.pdf" to tabular text.

Rem This comman depends upon the location of the input file
    Call pdftotext -eol dos -table "%Public%\Downloads\MMDL_S%YY%_SS_Sched.pdf" "%Temp%\%TeamFile%"
    If ERRORLEVEL 1 Exit /B

Rem Write left side of PDF file to the teams file
Rem while splitting right side of PDF file to a temporary teams file
    Set "GamesFile=MMDLgames.txt"

@Echo Off&SetLOCAL DisableDelayedExpansion &Rem.Third LOCAL scope to process poison characters.

    >"%Temp%\%LoadFile%" Rem.
(
    >%temp%\R%TeamFile% Rem.
    For /F "usebackq tokens=* delims=" %%F in (
        `Type %Temp%\%TeamFile% ^| findstr /i /r "^[1-9] ^[S][S]"`
    ) do (
        Set "rec=%%~F"
Rem     Peel right side to temporary team file before disturbing record with corrections
        >> %temp%\R%TeamFile% Call echo.%%rec:~106%%
Rem     Apply known left side record adjustments
        Call Set "recL=%%rec:~,106%%"
        Call Set "RecL=%%RecL:  = %%"
        Call Set "RecL=%%RecL: BYE= BYE, NO Match 00 No Location 888-000-0000%%"
        Call Set "RecL=%%RecL:SS SUPER A=SS A0%%"
Rem     Correct knowm typo(s)
        Call Set "RecL=%%RecL:t? KC=t?, KC%%"
        Call :strim recL recL
        Call echo=%%recL%%
    )
Rem
Rem Write right side of PDF file to the teams file
    For /F "usebackq tokens=* delims=" %%F in (
        `Type %Temp%\R%TeamFile% ^| findstr /i /r "^[1-9] ^[S][S]"`
    ) do (
        Set "recR=%%~F"
        For /L %%B in (1,1,3) Do Call Set "RecR=%%RecR:  = %%"
        Call Set "RecR=%%RecR: BYE= BYE, NO Match 00 No Location 888-000-0000%%"
        Call echo=%%recR%%
    )
) >>%Temp%\%LoadFile%

Rem Extract and Write games file saving first game date info
(
    For /F "usebackq tokens=* delims=" %%F in (
        `Type %Temp%\R%TeamFile% ^| findstr /vrib "[1-9] [S][S]"`
    ) do (
        Set "rec=%%~F"
        Call Set "recL=%%rec:~7,44%%"
        Call Set "recR=%%rec:~53%%"
        Call echo=%%recL%%
        If Not Defined FirstGame If Defined recR Call Set "FirstGame=%%recR%%"
    )
) >%Temp%\%GamesFile%

Rem Create date table using the contents of First Game: '%FirstGame%'
    For /F "tokens=2,4,5*" %%L in ("%FirstGame%") Do (
        If /I "%%~L" EQU "starts" (
            Set "mmm=%%~M"
            Call Set "mmm=%%mmm:~0,3%%"
            Set "dd=%%~N"
            Call Set "dd1=%%dd:~0,1%%"
            Call Set "dd2=%%dd:~0,2%%"
            Call 2>nul Set /A "dd=%%dd2%%" && (Call Set "dd=%%dd2%%") || (Call Set "dd=%%dd1%%")
        )
    )

Rem Define table of dates from
Rem Extracted First Game: '%MMM%' '%dd%' '%YYYY%'
    Set "WeeksFile=MMDLweeks.txt"

>> "%FireFile%" Echo.^<Week^>
(
    Set /a "ctrW=0"
    For /L %%D in (0,7,98) Do (
        Set /a "ctrW+=1"
        Call jtimestamp -d "'%MMM% %dd%, %YYYY%'" -f "{MM}/{DD}/{YYYY}" -c "od=%%~D" -R WeekDate
        Call Echo=%%WeekDate%%
    )
) >>%FireFile%
>> "%FireFile%" Echo.^</Week^>
>> "%FireFile%" Echo=

Rem Reprocess temporary load file
(
    For /F "usebackq tokens=1-2* delims= " %%A in (
        `Type "%Temp%\%LoadFile%"`
    ) do (
        If /I "%%~A" EQU "SS" (
            If Defined divTage (
                Call Echo=%%divTage%%
            )
            Set "divTag=^<%%~ADiv%%~B^>"
            Set "divTage=^</%%~ADiv%%~B^>"
            Call Echo=%%divTag%%
        ) Else (
            Echo.%%~A %%~B %%~C
        )
    )
) >>%FireFile%
>> "%FireFile%" Echo.%divTage%
>> "%FireFile%" Echo=

EndLOCAL & Rem End third LOCaL scope

Rem process pool games
(
    For /F "usebackq tokens=1-5,7,8,10,11,13  delims= " %%A in (
        `Type %Temp%\%GamesFile%`
    ) do (
        If /I "%%~B" EQU "Team" (
            Set "Games#=%%~A"
            If Defined GameFileName >> "%FireFile%" Echo.^</!GamefileName!^>
            Set "GameFileName=Sched%%~A"
            >> "%FireFile%" Echo.^<!GamefileName!^>
        )
        If /I "%%~C" EQU "At" (
            Set "rec=%%~A %%~B %%~D"
            >> "%FireFile%" Echo.!rec!
            Set "rec=%%~A %%~E %%~F"
            >> "%FireFile%" Echo.!rec!
            Set "rec=%%~A %%~G %%~H"
            >> "%FireFile%" Echo.!rec!
            If /I "!Games#!" EQU "8" (
                Set "rec=%%~A %%~I %%~J"
                >> "%FireFile%" Echo.!rec!
            )
        )
    )
)
>> "%FireFile%" Echo.^</%GamefileName%^>
>> "%FireFile%" Echo=

    >> "%FireFile%" Echo=
    >> "%FireFile%" Echo=%~nx0 generated table records above
    >> "%FireFile%" Echo=

Rem copy to user script from second GetSet label up to end of the script
    CScript //nologo //E:JScript "%~F0" ":GetSet2" /o:+1:+999 < "%~f0" >> "%FireFile%"
    Echo=
    Echo=%~nx0 copied %errorlevel% more records from itself to "%LoadFile%"

exit /B %errorlevel%

Rem begin replacement code block for FindREPL clone "MMDfire.bat"

:GetSet2
:GetSet
@echo off & setlocal enableextensions disabledelayedexpansion & Rem First LOCAL scope
Rem Start http://www.dostips.com/forum/viewtopic.php?p=54670#p54670
(set lf=^
%= BLANK LINE REQUIRED =%
)
set "cr=" & if not defined cr for /f "skip=1" %%C in (
'echo(^|replace ? . /w /u'
) do set "cr=%%C"

set ^"orig=  !random!  !  ^^!  ^^^^!  ^"^^  ^&^"^&  ^^^" %%os%%  ^"

call :strim res1 orig

@Echo Off&SetLOCAL EnableDelayedExpansion &Rem.Second LOCAL scope and keep environment uncluttered.
Rem Of all the gin joints, in all the towns, in all the world, she walks into mine
Rem Minute Man - World's largest Steel-Tip Dart League
Rem Start http://www.mmdl.org/

Rem Create prompt strings for division CHOICE commands
    For /F "tokens=* delims=v>" %%E in (
        'type "%~F0" ^| CScript //nologo //E:JScript "%~F0" "^<SSDiv" ^| Sort '
    ) DO (
        Set "Div=%%~E"
        Set "Div=!Div:~6,2!"
        Set "Div1=!Div:~0,1!"
        Set "Div2=!Div:~1,1!"
        If /I "!Div!" EQU "A0" (
            Set "DivV=!Div!
            Set "DivA=S"
            Set "Div#="
            Set "DivP=S[for SUPER A]
        ) Else (
            Set "DivV=!DivV!!Div!
            Set "DivC=!DivA:~-1!"
            If "!DivC!" NEQ "!Div1!" Set "DivA=!DivA!!Div1!"
            If Defined Div# (
                For %%H in (!Div2!) Do Set "DivL=!Div#:%%~H=!"
                If "!DivL!" EQU "!Div#!" Set "Div#=!Div#!!Div2!"
            ) Else (
                Set "Div#=!Div2!"
            )
            Set "DivP=!DivP! !Div!"
        )
    )

    Echo=The board is set to ask for division and team information.
    Echo=
    Echo=Enter four or more letters to search the team lists.
    Echo=Try team, establishment, street, town or city names.
    Echo=Some suggestions may be offered for division selection.
    Echo=Less characters will transfer directly to prompting.
    Echo=
    Set /P "Hint=Press optional search characters and the 'Enter' key: "

Rem Prompt for Division name
:RePromptDiv
CLS
    Call :HintSearch "%Hint%"
    Echo=
    Echo=South Shore Division names starting with SS:
    Echo=  !DivP! (or X to eXit^)
    Set <Nul /P "=Throw a first dart at a alphabetic character: SS"
    CHOICE /C X%DivA% /N
    Set /A "DivPos=%ErrorLevel%-2"
    If /I "%ErrorLevel%" EQU "1" (Echo=Exiting on request. &Exit /B)
    If /I "!DivPos!" EQU "0" (
        Set "MyDiv=A0"
        Goto Divided
    )
    Set "DivA=!DivA:~%DivPos%,1!"
CLS
    Call :HintSearch "%Hint%"
    Echo=
    Echo= South Shore Division names starting with SS:
    Echo= !DivP! (or X to eXit^)
    Set <Nul /P "=Throw the second dart at a numeric character: SS%DivA%"
    CHOICE /C X%Div#%  /N
    Set /A "DivPos=%ErrorLevel%-2"
    If /I "%ErrorLevel%" EQU "1" (Echo=Exiting on request. &Exit /B)
    Set "Div#=!Div#:~%DivPos%,1!"
    Set "MyDiv=%DivA%%Div#%"
:Divided

Rem Validate the selection
    For %%a in (%MyDiv%) do Set "htest=!DivV:%%a=!"
    If /I "!hTest!" EQU "%DivV%" (
        Echo.'%MyDiv%' was not from the list.  Please try again.
        Echo.
        GoTo :RePromptDiv
    ) Else (
        Set "tag=SSDiv%MyDiv%"
        Echo=
        Call :TeamArray rem
    )

    Echo=
    Echo=List of teams in the SS%MyDiv% division:
    For /L %%L In (1,1,%Name[0]%) Do (
        Echo= %%L. '!Name[%%L]!' from '!Home[%%L]!', !Addr[%%L]!
    )
    Set <Nul /P "=Enter a team number for the three dart out (or X to eXit) "
    CHOICE /C %NameP%X
    Set "MyTeam=%errorlevel%"
    Set /A "NameC=%errorlevel%+1"
    If ErrorLevel %NameC% (Echo=Exiting on request. &Exit /B)

    Echo=
    Echo=%MyTeam%. '!Name[%MyTeam%]!' plays out of '!Home[%MyTeam%]!' at !Addr[%MyTeam%]!.
    Echo=Now ready to process the %Team#%-Team Schedule for team %MyTeam%. in the SS%MyDiv% division.

    Echo=
    Set "tag=Week"
    Call :TableArray Rem

    Set "tag=Sched%team#%"
    Call :TableArray rem

Rem Combine pool teams Dates and games
Rem For Minute Man Dart League - HOME TEAM IS ON THE RIGHT

    Set    "file=MMD\team%MyTeam%_Sched.txt"
    Set "PktFile=MMD\team%MyTeam%_PktSched.txt"
    If Not Exist MMD MD MMD
    Echo.
    Echo Creating or replacing contents of:
    Echo.    "%file%"
    Echo.and "%PktFile%"
    >%PktFile% Rem.

(
    Echo.Minute Man Dart League Division Schedule %MYDiv%
    Set "qtr= %MyTeam%. !name[%MyTeam%]!"
    Call :quarters line qtr ""
    >>%PktFile% Echo.!line!
    Echo.Spring 2018 - for team %MyTeam%. !name[%MyTeam%]! from !home[%MyTeam%]!
    Set "qtr= MMDL 02/04 base"
    Call :quarters line qtr
    >>%PktFile% Echo.!line!
    Set "qtr=            "
    Call :quarters line qtr
    >> %PktFile% Echo.!line!
    Echo.Based on MMDL web site Schedule 02/04/2018
    Echo.Customization courtesy of John A. of the Braintree Moose
rem Set "Current=%%~G"
    Echo.

    For /F "tokens=1-3" %%M in (
        'Type "%~f0" ^| CScript //nologo //E:JScript "%~F0" "^<%tag%>" /E:"^</%tag%>" /o:+1:-1'
    ) DO (
        Set "Match=%%~M"
        Set "Guest=%%~N"
        Set "Host=%%~O"
        Set "When=!Week[%%~M]:~,10!"
        Set "sWhen=!When:~,5!"
        For %%G In (!Guest!) Do (
            For %%H In (!Host!) Do (
                If /I "!Host!" EQU "%MyTeam%" (
                    Echo.
                    Echo.[!When!] HOME at !home[%%~H]!
                    If /I "!name[%%~W]!" EQU "Bye" (
                        Echo.NO game for %%~H. !name[%%~H]!
                    ) Else (
                        Echo.Hosting %%~G. !name[%%~G]! from !home[%%~G]!
                    )
                    >>%PktFile% (
                        If /I "!name[%%~G]!" EQU "Bye" (
                            Set "qtr=!sWhen! Bye 
                            Call :quarters line qtr ""
                            Echo.!line!
                        ) Else (
                            Set "sName=!Name[%%G]:~0,7!"
                            Set "qtr=!sWhen! HOME !sName!"
                            Call :quarters line qtr
                            Echo.!line!
                        )
                    )
                ) ELse If /I "!Guest!" EQU "%MyTeam%" (
                    Echo.
                    Set "sName=!Home[%%H]:~0,7!"
                    Set "qtr=!sWhen! !sName! %%~H"
                    Call :quarters line qtr " "
                    >>%PktFile% Echo.!line!
                    Set "sName=!name[%%H]:~0,13!"
                    Set "qtr=!sName! "
                    Call :quarters line qtr ""
                    >>%PktFile% Echo.!line!
                    Echo.[!When!] AWAY at "!Home[%%~H]!" vs. %%~H. !name[%%~H]!
                    Echo.AWAY game at !Addr[%%~H]!
                ) ELse (
                    Rem Echo %MyTeam% is not involved
                )
            )
        )
    )
    Set "qtr=..            .."
    Call :quarters line qtr
    >> %PktFile% Echo.!line!
    Set "qtr= Pocket Edition"
    Call :quarters line qtr ""
    >> %PktFile% Echo.!line!
) > %file%

Rem Splash completion display and perhaps
Rem drop into default editor printing instructions here
Rem Echo.
Rem Type  %file%
Rem Start %PktFile%
Rem Start %file%
    Start MMD

Rem >Nul Pause

    Exit /B

:TableArray
    Set "Do=%~1"
    If not defined Do Set "Do=Set"
    Set /A "CtrW=0"
    For /F "tokens=1* delims= " %%W In (
        'Type "%~f0" ^| CScript //nologo //E:JScript "%~F0" "^<%tag%>" /E:"^</%tag%>" /o:+1:-1'
    ) DO (
        Set /A "CtrW+=1"
        For %%H in (!CtrW!) Do Set %tag%[%%H]=%%~W %%~X
    )
    Set /A  "%tag%[0]=%CtrW%"
    Set "%tag%{Rem}=Echo=The array for the <%tag%> table is now set to !%tag%[0]! entries."
    !%tag%{Rem}!
Rem Display of %tag%[ is optional
    %Do% %tag%[
Exit /B %errorlevel%

:HintSearch
Rem Display search results for division CHOICEs
    Set "Div="
    Set /A "CtrH=0"
    If NOT Defined Hint Exit /B
    Set "Hint=%Hint%   "
    If /I "%Hint:~3,1%" EQU " " Exit /B
    Call :strim Hint Hint
    Echo=Only the first instance of a team related to '%~1' is shown for any division.
Rem The FOR command pipes itself to isolate the tables and avoid searching actual script code
    For /F "tokens=1* delims=," %%E in (
        'type "%~F0" ^| CScript //nologo //E:JScript "%~F0" /I "^<SSDiv" /E:"/SSDiv" /o:+0:-0 ^| CScript //nologo //E:JScript "%~F0" /I "%~1" /E:"/SSDiv" /o:+0:-0'
    ) DO (
        If Not Defined Hints (
            Set "Hint2=%%~F"
            Set "Hint2=!Hint2:~,30!"
            Set "Hints=%%~E. !Hint2!"
        ) ELSe (
            Set "Div=%%~E"
            Set "Div=!Div:~2,5!"
            If /I "!Div!" EQU "SSDiv" (
                Set /A "CtrH+=1"
                Set "HintD=%%E"
                Set "HintD=Div=!HintD:~2,2!!HintD:~7,2!"
                Echo=%%E !HintD! !Hints!
                Set "Hints="
            )
        )
    )
    If Not Defined Div (
        Echo=No suggestions to offer related to '%~1'.
    ) ELSE (
        Echo=Listed %CtrH% suggestions related to '%~1'.
    )
Exit /B %errorlevel%

:TeamArray
    Set "Do=%~1"
    If not defined Do Set "Do=Set"
    Set /A "CtrW=0"
(
    For /F "tokens=1* delims=," %%W In (
        'Type "%~f0" ^| CScript //nologo //E:JScript "%~F0" "^<%tag%>" /E:"^</%tag%>" /o:+1:-1'
    ) DO (
        Set /A "CtrW+=1"
        Set "NameP=!NameP!!CtrW!"
        For %%H in (!CtrW!) Do (
            Set "Name=%%~W"
            Set "Name[%%H]=!Name:~2!"
            Echo=%%~X
        )
    )
) > %Temp%\MMDvenues.txt
    Set /A  "Name[0]=%CtrW%"
    Set "Name{Rem}=Echo=The Name array for the <%tag%> table is now set to !Name[0]! entries."
    !Name{Rem}!
Rem Display of Name[ is optional
    %Do% Name[

Rem Define array of team home establishments and addresses
    Set "CapS=ABCDEFGHIJKLMNOPQRSTUVWXYZ"

    Set /A "Team#=%CtrW%"
    Set /A "CtrW=0"
    For /F "usebackq tokens=1* delims=-" %%F in (
        `Type %Temp%\MMDvenues.txt`
    ) do (
        Set /A "CtrW+=1"
        Set "RecB=%%~F"
        Set "RecB=!RecB:~,-3!"
        Call :strim RecB RecB
        Set "RecAddr="
        Set "RecHome="
        >Nul Call :ArgCtr !recB!
        For /L %%L In (1,1,!ArgCnt!) Do (
            Set "ch=!arg[%%L]!"
            If Defined RecAddr (
                Set "RecAddr=!RecAddr! !ch!"
            ) eLSE (
                Set "test1=!ch:~0,1!"
                For %%a in (!test1!) do Set "test=!caps:%%a=!"
                If /I "!test!" EQU "%CapS%" (
                    Set "RecAddr=!ch!"
                )
            )
            If Not Defined RecAddr (
                Set "RecHome=!RecHome! !ch!"
            )
        )
        Set "Home[!CtrW!]=!recHome:~1!"
        Set "Addr[!CtrW!]=!recAddr!"
    )

    Set /A  "Home[0]=%CtrW%"
    Set "Home{Rem}=Echo=The Home array for the <%tag%> table is now set to !Home[0]! entries."
    !Home{Rem}!
Rem Display of Home[ is optional
    %Do% Home[

    Set /A  "Addr[0]=%CtrW%"
    Set "Addr{Rem}=Echo=The Addr array for the <%tag%> table is now set to !Addr[0]! entries."
    !Addr{Rem}!
Rem Display of Addr[ is optional
    %Do% Addr[

Exit /B %errorlevel%

:Quarters result= original=
:: trims leading and trailing whitespace from a string
setlocal enabledelayedexpansion
set "die=" & if not defined %2 (>&2 echo(var "%2" not defined
set "die=1") elSE set "str=!%2!"

if not defined die for %%L in ("!lf!") ^
do if "!str!" neq "!str:%%~L=!" (
>&2 echo(var "%2" contains linefeeds & set "die=1")

if not defined die for %%C in ("!cr!") ^
do if "!str!" neq "!str:%%~C=!" (
>&2 echo(var "%2" contains carriage returns & set "die=1")

if not defined die (for /f eol^= %%A in ("!str!") do rem nop
) || (>&2 echo(var "%2" consists entirely of whitespace
set "die=1")

if defined die goto die

    Set "sep=%~3"
    If Not Defined sep Set "sep="

    set "str=!str!           "
    set "str=!str:~0,17! !Sep! "
    set "str=!str!!str!!str!!str!"

(endlocal & set "%1=%str%")
exit /b 0

:die
endlocal & set "%1=" & exit /b 1

Rem end of replacement code block for FindREPL clone "MMDfire.bat"



Rem batch subroutines


:ArgCtr
set argCnt=1
:getArgs
Rem Posted: Sun Jan 22, 2012 9:02 pm by dbenham
Rem Post subject: Re: foolproof counting of arguments
Rem Start http://www.dostips.com/forum/viewtopic.php?p=13002#p13002
Rem this[ArgCtr.bat] cloned from Getargs.bat Nov 2,2017 by thefeduke
>"%temp%\getArg.txt" <"%temp%\getArg.txt" (
  setlocal disableExtensions
  set prompt=#
  echo on
  for %%a in (%%a) do rem . %1.
  echo off
  endlocal
  set /p "arg[%argCnt%]="
  set /p "arg[%argCnt%]="
  set "arg[%argCnt%]=!arg[%argCnt%]:~7,-2!"
  if defined arg[%argCnt%] (
    set /a argCnt+=1
    shift /1
    goto :getArgs
  ) else set /a argCnt-=1
)
del "%temp%\getArg.txt"
Exit /B


:strim result= original=
:: trims leading and trailing whitespace from a string
setlocal
set "ddx=!"
setlocal enabledelayedexpansion
set "die=" & if not defined %2 (>&2 echo(var "%2" not defined
set "die=1") else set "str=!%2!"

if not defined die for %%L in ("!lf!") ^
do if "!str!" neq "!str:%%~L=!" (
>&2 echo(var "%2" contains linefeeds & set "die=1")

if not defined die for %%C in ("!cr!") ^
do if "!str!" neq "!str:%%~C=!" (
>&2 echo(var "%2" contains carriage returns & set "die=1")

if not defined die (for /f eol^= %%A in ("!str!") do rem nop
) || (>&2 echo(var "%2" consists entirely of whitespace
set "die=1")

if defined die goto die

set "str=!str:^=^^^^!"
set "str=!str:"=""!"
set "str=%str:!=^^^!%" !

call :_strim "%%str%%

if not defined ddx set "str=!str:^=^^^^!"
if not defined ddx set "str=%str:!=^^^!%" !
set "str=!str:""="!"

for /f tokens^=*^ eol^= %%A in ("!str!") do (
endlocal & endlocal & set "%1=%%A" !)
exit /b 0

:die
endlocal & endlocal & set "%1=" & exit /b 1

:_strim
:: subfunction
set "str=%~1" !
exit /b 0


Rem end of batch subroutines

:showUsage
setlocal EnableDelayedExpansion

< "%~F0" CScript //nologo //E:JScript "%~F0" "^<usage>" /E:"^</usage>" /O:+1:-1
set n=1
set "choices="
for %%a in ("http://www.dostips.com/forum/viewtopic.php\Qf=3&t=4697"
            "http://www.dostips.com/forum/viewtopic.php\Qf=3&t=4697&p=38121#p38121"
            "http://msdn.microsoft.com/en-us/library/6wzad2b2(v=vs.84).aspx"
            "http://msdn.microsoft.com/en-us/library/kstkz771(v=vs.84).aspx"
            "http://msdn.microsoft.com/en-us/library/ce57k8d5(v=vs.84).aspx"
            "http://msdn.microsoft.com/en-us/library/htbw4ywd(v=vs.84).aspx") do (
   set "choices=!choices!!n!"
   set "option[!n!]=%%~a"
   set /A n+=1
)

:getOption
echo/
choice /C %choices%0 /N /M "Select one of previous help topics:"
if errorlevel %n% goto :EOF
set "choice=%errorlevel%"
explorer "!option[%choice%]:\Q=?!"
echo  - Help on topic %choice% opened...
goto getOption

End of Batch section


@end


// JScript section



// PARSE PARAMETERS


// FINDREPL [/I] [/V] [/N] rSearch [/E:rEndBlk] [/O:s:e] [/B:rBlock] [/$:1:2...]
//          [[/R] [/A] sReplace] [/Q:c] [/S:source]
//          [/J[:n] [/L:jEnd]] [/G:file]

var options = WScript.Arguments.Named,
    args    = WScript.Arguments.Unnamed,
    env     = WScript.CreateObject("WScript.Shell").Environment("Process"),
    fso     = new ActiveXObject("Scripting.FileSystemObject"), file,

    ignoreCase   = options.Exists("I")?"i":"",
    notMatched   = options.Exists("V"),
    showNumber   = options.Exists("N"),
    search       = undefined,
    endBlk       = undefined,
    offset       = undefined,
    block        = undefined,
    submatches   = undefined,
    justReplaced = options.Exists("R"),
    alternation  = options.Exists("A"),
    replace      = undefined,
    quote        = options.Item("Q"),
    Jexpr        = options.Exists("J"),

    lineNumber = 0, range = new Array(),
    procLines = false, procBlocks = false,
    nextMatch, result  = 0,

    match = function ( line, regex ) { return line.search(regex) >= 0; },

    parseInts =
       function ( strs ) {
          var Ints = new Array();
          for ( var i = 0; i < strs.length; ++i ) {
             Ints[i] = parseInt(strs[i]);
          }
          return Ints;
       },

    getRegExp =
       function ( param, justLoad ) {
          var result = param;
          if ( result.substr(0,1) == "=" ) {
             result = env(result.substr(1));
          } else {
             if ( quote != undefined ) result = result.replace(eval("/"+quote+"/g"),'"');
          }
          if ( ! justLoad ) result = new RegExp(result,"gm"+ignoreCase);
          return result;
       }
    ; // end var

// Predefined variables

for ( var i = 1; block=options.Item("ARG"+i); i++ ) {
   if ( block.substr(0,1) == "=" ) block = env(block.substr(1));
   eval ( 'var ARG'+i+'="'+block+'";' );
}
var daysNow = Math.floor((new Date()).getTime()/86400000), n = 0;
if ( Jexpr) {
   var JexprN = 10;
   if ( options.Item("J") ) JexprN = parseInt(options.Item("J"));
   for ( var i = 1; i <= JexprN; i++ ) eval( "var SUM"+i+"=0,N"+i+"=0,PROD"+i+"=1;" );
}

// Predefined functions

function choose(arg,i){return(arg[i]);};
function max(arg,a,b){var val=-Number.MAX_VALUE; for ( var i=a; i<=b; i++ ) if ( arg[i]>val ) val=arg[i]; return(val);};
function min(arg,a,b){var val= Number.MAX_VALUE; for ( var i=a; i<=b; i++ ) if ( arg[i]<val ) val=arg[i]; return(val);};
function hlookup(arg,lim,a,b){var ind=a; for ( var i=a; i<=b; i++ ) if ( arg[i]>arg[ind] && arg[i]<=lim ) ind=i; return(ind);};
function sum(arg,a,b){var val=0,v; for ( var i=a; i<=b; i++ ) { val+=v=parseFloat(arg[i]); eval("SUM"+i+"+=v;++N"+i+";"); } n=b-a+1; return(val);};
function prod(arg,a,b){var val=1,v; for ( var i=a; i<=b; i++ ) { val*=v=parseFloat(arg[i]); eval("PROD"+i+"*=v;"); } n=b-a+1; return(val);};
function days(s){return(daysNow - Math.floor((new Date(s)).getTime()/86400000));};
function prompt(s){WScript.Stderr.Write('Replace "'+s+'" by: '); return(file.ReadLine());};

if ( file = options.Item("G") ) {
   file = fso.OpenTextFile(file, 1);
   search = "";
   if ( alternation ) replace = "";
   var sepIn = (Jexpr?'||':'|'), sepOut = "";
   while ( ! file.AtEndOfStream ) {
      block = file.ReadLine().split(sepIn);
      search += sepOut+block[0];
      if ( alternation ) replace += sepOut+block[1];
      sepOut = sepIn;
   }
   block = undefined;
   file.Close();
} else {  // No option /G given
   if ( args.length > 0 ) {
      search = getRegExp(args.Item(0),true);
   }
   if ( args.length > 1 ) {
      replace = args.Item(1);
      if ( replace.substr(0,1) == "=" ) replace = env(replace.substr(1));
   }
}
if ( replace ) {  // Check this! It may cause problems when combined with /A and /J
   // replace = eval('"' + replace + '"');
   if ( quote != undefined ) replace = replace.replace(eval("/"+quote+"/g"),'"');
}


if ( options.Exists("E") ) {
   endBlk = getRegExp(options.Item("E"));
   procBlocks = true;
}
if ( options.Exists("O") ) {
   offset = parseInts(options.Item("O").split(":"));
   procBlocks = true;
}
if ( options.Exists("B") ) {
   block = getRegExp(options.Item("B"),true);
}
if ( options.Exists("$") ) submatches = parseInts(options.Item("$").split(":"));
var removeCRLF = false;


if ( replace != undefined ) {
   removeCRLF = (block == "\\r\\n") && (replace == "");

   if ( alternation ) {  // Enable alternation replacements from "Se|ar|ch" to "Re|pla|ce"
      if ( ! Jexpr ) {  // Original version

         var searchA = search.split("|"),
             replaceA = replace.split("|"),
             repl = new Array();
         for ( var i = 0; i < searchA.length; i++ ) {
            repl[searchA[i]] = replaceA[i];
         }
         replace = function($0,$1,$2) { return repl[$0]; };
         searchA.length = 0;
         replaceA.length = 0;

      } else {  // Version 2: Search alternation have regular expressions:  "regexp1||regexp2||regexp3"
                //            Replace alternation have JScript expressions: "'Re'||'pla'||'ce'"

         Jexpr = false;

         var searchA = search.split("||"),      // divide search "regexp1||regexp2||regexp3" in parts
             repl = replace.split("||");      // the same for "replace1||replace2||replace3"
         search = "";
         replace = "$0,";
         for ( var i = 0; i < searchA.length; i++ ) {   // process each regexpI
            search += (i?'|':'')+'('+searchA[i]+')';   // re-assemble search regexp as "(regexp1)|(regexp2)|(regxep3)"
            var subexprs = searchA[i].match(/[^\\]?\(/g);// count subexpressions in this regexpI
            subexprs = subexprs?subexprs.length:0;   // zero if no one
            for ( var j = 0; j <= subexprs; j++ ) {   // insert each subexpression
                  replace += '$'+(i+1)+'$'+j+',';   // 'function($0,' + "$i$0,$i$j,..."
            }
            repl[i] = repl[i].replace(/\$(\d{1,2})/g,"$$"+(i+1)+"$$$1");   // change "$n" by "$i$n" in replaceI
         }

         replace += "$offset,$string";             // 'function($0, $i$0,$i$1,$i$2, ...' + "$offset, $string)"
         eval ( "replace = function("+replace+") { for (var i=1; !eval('$'+i+'$0'); i++); return(eval(repl[i-1])); };" );
         searchA = undefined;
         if ( subexprs ) subexprs = undefined;

      }

   } else {  // No alternation

      var evalReplace = "";
      if ( Jexpr) {
         for ( var i = 0; i <= JexprN; i++ ) evalReplace += (i?',':'')+'$'+i;
         eval ( "evalReplace = function("+evalReplace+") { return(eval(replace)); };" );
      }

   }

} else {  // replace == undefined: Find-only operation (to do: adjust values of /$ switch)

   if ( Jexpr ) {  // Find a "second-level" alternation
      var searchA = search.split("||");         // divide search "regexp1||regexp2||regexp3" in parts
      search = "";
      for ( var i = 0; i < searchA.length; i++ ) {
         search += (i?'|':'')+'('+searchA[i]+')';   // re-assemble search regexp as "(regexp1)|(regexp2)|(regxep3)"
      }
      searchA = undefined;
   }

}

if ( search != undefined ) search = new RegExp(search, "gm"+ignoreCase);
if ( block  != undefined ) block  = new RegExp(block , "gm"+ignoreCase);
file = fso.OpenTextFile("CONIN$", 1);


// PROCESS INPUT FILE


// FINDREPL [/I] [/V] [/N] rSearch [/E:rEndBlk] [/O:s:e] [/B:rBlock] [/$:s1...]
//          [[/R] [/A] sReplace] [/Q:c] [/S:sSource]

//          In Search and Replace operations:
//            /V or /N switches implies line processing
//            /E or /O switches implies block (and line) processing
//          If Search operation (with no previous switches) have NOT /$ switch:
//            implies line processing (otherwise is file processing)

if ( options.Exists("S") ) {  // Process Source string instead of file
   var source = options.Item("S");
   if ( source.substr(0,1) == "=" ) source = env(source.substr(1));
   var fileContents = new Array(), lastLine = 1;
   fileContents[0] = source;
   procLines = true;
} else {  // Process Stdin file

   fileContents = WScript.StdIn.ReadAll();

   if ( notMatched || showNumber || procBlocks ) procLines = true;
   if ( replace==undefined && submatches==undefined ) procLines = true;

   if ( procLines ) {  // Separate file contents in lines
      var lastByte = fileContents.slice(-1);
      fileContents = fileContents.replace(/([^\r\n]*)\r?\n/g,"$1\n").match(/^.*$/gm);
      lastLine = fileContents.length - ((lastByte == "\n")?1:0);
   }

   if ( procBlocks ) {  // Create blocks of lines
      if ( search != undefined ) {  // Blocks based on Search lines:
         if ( offset == undefined ) offset = new Array(0,0);
         for ( var i = 1; i <= lastLine; i++ ) {
            if ( match(fileContents[i-1],search) ) {
               if ( endBlk != undefined ) {  // 1- from Search line to EndBlk line [+offsets].
                  for ( var j=i+1; j<=lastLine && !match(fileContents[j-1],endBlk); j++ );
                  if ( j <= lastLine ) {
                     var s = i+offset[0], e = j+offset[1];
                     // Insert additional code here to cancel overlapped blocks
                     range.push(s>0?s:1, e>0?e:1);
                  }
                  i = j;
               } else {  // 2- surrounding Search lines with offsets.
                  s = i+offset[0], e = i+offset[1];
                  range.push(s>0?s:1, e>0?e:1);
               }
            }
         }
      } else {  // Offset with no Search: block is range of lines
         if ( offset.length < 2 ) offset[1] = lastLine;
         s = offset[0]<0 ? offset[0]+lastLine+1 : offset[0];
         e = offset[1]<0 ? offset[1]+lastLine+1 : offset[1];
         range.push(s>0?s:1, e>0?e:1);
      }
      if ( range.length == 0 ) WScript.Quit(0);
      range.push(0xFFFFFFFF,0xFFFFFFFF);
   }

}
// endif Process Source string instead of file

if ( replace == undefined ) {  // Search operations
   if ( procLines ) {  // Search on lines
      if ( procBlocks ) {  // Process previously created blocks
         for ( var r=0, lineNumber=1; lineNumber <= lastLine; lineNumber++ ) {
            if ( (range[r]<=lineNumber && lineNumber<=range[r+1]) != notMatched ) {
               if ( submatches != undefined ) {
                  if ( showNumber ) WScript.Stdout.Write(lineNumber+":");
                  while ( (nextMatch = block.exec(fileContents[lineNumber-1])) != null ) {
                     for ( var s = 0; s < submatches.length; s++ ) {
                        WScript.Stdout.Write(" " + (quote!=undefined?quote:'"') +
                                                   nextMatch[submatches[s]] +
                                                   (quote!=undefined?quote:'"'));
                     }
                     result++;
                  }
                  WScript.Stdout.WriteLine();
               } else {
                  if ( block == undefined  ||  match(fileContents[lineNumber-1],block) ) {
                     if ( showNumber ) WScript.Stdout.Write(lineNumber+":");
                     WScript.Stdout.WriteLine(fileContents[lineNumber-1]);
                     result++;
                  }
               }
            }
            if ( lineNumber >= range[r+1] ) r += 2;
         }
      } else {  // Process all lines for Search
         for ( lineNumber = 1; lineNumber <= lastLine; lineNumber++ ) {
            if ( match(fileContents[lineNumber-1],search) != notMatched ) {
               if ( showNumber ) WScript.Stdout.Write(lineNumber+":");
               if ( submatches != undefined ) {
                  search.lastIndex = 0;
                  while ( (nextMatch = search.exec(fileContents[lineNumber-1])) != null ) {
                     for ( var s = 0; s < submatches.length; s++ ) {
                        WScript.Stdout.Write(" " + (quote!=undefined?quote:'"') +
                                                   nextMatch[submatches[s]] +
                                                   (quote!=undefined?quote:'"'));
                     }
                     result++;
                  }
                  WScript.Stdout.WriteLine();
               } else {
                  WScript.Stdout.WriteLine(fileContents[lineNumber-1]);
                  result++;
               }
            }
         }
      }

   } else {  // Search on entire file and show submatched substrings
      if ( submatches != undefined ) {
         while ( (nextMatch = search.exec(fileContents)) != null ) {
            for ( var s = 0; s < submatches.length; s++ ) {
               WScript.Stdout.Write(" " + (quote!=undefined?quote:'"') +
                                          nextMatch[submatches[s]] +
                                          (quote!=undefined?quote:'"'));
            }
            result++;
            WScript.Stdout.WriteLine();
         }
      }
   }

} else {  // Replace operations

   if ( procLines ) {  // Replace on lines
      if ( procBlocks ) {  // Process previously created blocks
         if ( block == undefined ) block = search;  // Replace rSearch or rBlock (the last one)
         var CRLFremoved = false;
         for ( var r=0, lineNumber=1; lineNumber <= lastLine; lineNumber++ ) {
            if ( range[r]<=lineNumber && lineNumber<=range[r+1] ) {
               if ( removeCRLF ) {
                  WScript.Stdout.Write(fileContents[lineNumber-1]);
                  CRLFremoved = true;
                  result++;
               } else {
                  if ( match(fileContents[lineNumber-1],block) ) {
                     if ( CRLFremoved ) { WScript.Stdout.WriteLine(); CRLFremoved = false; }
                     WScript.Stdout.WriteLine(fileContents[lineNumber-1].replace(block,Jexpr?evalReplace:replace));
                     result++;
                  } else {
                     if ( CRLFremoved ) { WScript.Stdout.WriteLine(); CRLFremoved = false; }
                     if ( ! justReplaced ) WScript.Stdout.WriteLine(fileContents[lineNumber-1]);
                  }
               }
            } else {
               if ( CRLFremoved ) { WScript.Stdout.WriteLine(); CRLFremoved = false; }
               if ( ! justReplaced ) WScript.Stdout.WriteLine(fileContents[lineNumber-1]);
            }
            if ( lineNumber >= range[r+1] ) r += 2;
         }
         if ( CRLFremoved ) { WScript.Stdout.WriteLine(); CRLFremoved = false; }
      } else {  // Process all lines for Replace
         for ( lineNumber = 1; lineNumber <= lastLine; lineNumber++ ) {
            if ( match(fileContents[lineNumber-1],search) ) {
               WScript.Stdout.WriteLine(fileContents[lineNumber-1].replace(search,Jexpr?evalReplace:replace));
               result++;
            } else {
               if ( ! justReplaced ) WScript.Stdout.WriteLine(fileContents[lineNumber-1]);
            }
         }
      }

   } else {  // Replace on entire file
      WScript.Stdout.Write(fileContents.replace(search,Jexpr?evalReplace:replace));
   }

   if ( lastLine = options.Item("L") ) {
      if ( lastLine.substr(0,1) == '=' ) lastLine = env(lastLine.substr(1));
      WScript.Stdout.WriteLine( eval(lastLine) );
   }

}
WScript.Quit(result);
